This is a work on progress ansible playbook for deploying a ceph cluster.


inventory.ini: This is the inventory file that Ansible will use when running the playbooks. There are several groups within this playbook.

ansible_node: This is the host that the ansible playbooks are being executed on.
- mons: These are the hosts that will serve as monitor nodes.
- mgrs: These are the hosts that will serve as manager nodes.
- osds: These are the nodes that will host object storage daemons.
- mdss: These are the nodes that will serve as RADOS gateway nodes.
Add the relevant nodes into their groups using the format "HOSTNAME ansible_host=IP ADDRESS"

ceph.conf.j2: This is the template file that will be used to create the ceph.conf file used by all nodes in the cluster.

group_vars: The files in this directory contain variables that are to be applied to hosts within the designated groups in the inventory file. The all.yml file contains variables that are applied to all hosts regardless of their specific group. This file contains the following important variables:

- public_network: This is the network that the nodes in the cluster use to communicate with other network devices.
- cluster_network: This is the internal network that the nodes in the cluster use to communicate with each other.
- mon_initial_hosts: This contains the hostnames of the devices meant to be monitor nodes when the cluster is first created. This is important when creating the cluster and is important when executing tasks in ansible. Make sure that the hostnames set here match the hostnames used for that IP address in the inventory.ini file.
- ceph_repo: This contains the link to the Ceph repo to be used in the cluster.
- ceph_noarch_repo: This contains the Ceph noarch repo to be used in the cluster. Should be the exact same release as the ceph_repo link.
- ceph_ubuntu_repo: This contains the link to the Ceph repo used by Ubuntu hosts in the cluster.
- ipv4_status: This defines whether IPv4 addressing is used in the cluster. It defaults to true if not set. Set this to false if only using IPv6.
- ipv6_status: This defines whether IPv6 addressing is used in the cluster. Set to false if it's not being used or true if it is being used.
- pool_name: These variables set the names of the pools to be used by a filesystem in the cluster.
- filesystem_name: This variable sets the name of the filesystem to be created and used in the cluster.
- pg_num: These variables set the number of placement groups the corresponding pool with have.
- pool_type: These variables define whether the pool is replicated of erasure-coded.
- crush_rule: These variables define what CRUSH rule the corresponding pool will use. Can be left blank to use the default rule.
- additional_pools: This variable determines whether three additional pools will be added when running the cephfs playbook. Set to true to enable.
The mons.yml file also contains important variables.

new_mons: These are the hostnames of mons added to the cluster after the initial set of mons have been added. Like the hostnames in mon_initial_hosts, it's important for these hostnames to match the hostnames corresponding to that IP address in the inventory.ini file.
combined_mons: This variable represents the combined number of values in mon_initial_hosts and new_mons and is used to prevent users from adding an even number of mons to the cluster. Mons require a quorum, or majority in order to function properly. As such, clusters with an even number of mons in which half the mons have gone down can create serious problems in the cluster.

host_vars: The files in this directory are similar to the files in group_vars, but instead relate to the hostnames set in the inventory.in file. There are two important variables that can be set in these files:

- dedicated_device_db: This is a bool value (true or false) that represents whether this node will use a dedicated device for storing some osd data.
- dedicated_devices: This is a list of devices that will be used as dedicated storage for some data relating to osds such as metadata.


PLAYBOOKS

The following section will break down the various playbooks found within this repo. These playbooks are intended to be modular and allow for selective usage if the user only needs to use a select few of them (ex: adding new osds to a cluster)

ceph_prerequisites.yml: This playbook checks for the existence of the ceph.conf file in /etc/ceph. If it doesn't exist, it creates both the file and the /etc/ceph directory if they do not exist on any host in the inventory.ini file. The ceph.conf file is created on the host running the playbook and then copies it to every other node that doesn't have a ceph.conf file. Finally, it checks if the repositories for ceph are present on the hosts and, if not, creates the Ceph repos on all the hosts that do not have them.

ceph_packages.yml: This playbook downloads the ceph packages onto the hosts. There's two versions; ceph_packages_rocky installs ceph packages on Rocky Linux hosts and ceph_packages_ubuntu installs ceph packages on Ubuntu hosts. All hosts will install ceph-common while the rest of the packages are installed depending on what group in the inventory file each host is a part of.

ceph_keyring_setup.yml: This playbook creates the keyring that is required for ceph operations. It creates the key for monitor nodes and the client admin key. It then distributes the keyring to every other host that does not have the keyring already.

mon_count_check.yml: This playbook contains checks meant to fail the playbook of there's an issue with the [mons] inventory group and/or the mon_initial_hosts, new_mons, and combined_mons variables. This playbook will fail under the following conditions:

- There's an even number of entries in the [mons] inventory group and/or the combined_mons variable.
- The number of entries in the [mons] inventory group and combined_mons variable don't match.
- There's a hostname mismatch between the hostnames listed in the inventory file and the combined_mons variable (ex. the inventory file has entries for node1, node2, and node3 in the [mons] group while the entries in the combined_mons variable include node1, node2, and node4.)

add_mons.yml: This playbook creates the monitor nodes on hosts within the mons group in the inventory file. It can be used to create mons when creating the cluster and it can be used to add new mons to the cluster.

add_mgrs.yml: This playbook is used to create manager nodes on hosts within the mgrs group in the inventory.ini file. These hosts create their mgr keyrings if they don't already exist.

filesystem_check.yml: This playbook is meant to prevent attempts at creating osds using devices with filesystems already present. This playbook will fail if a filesystem is detected on any of the drive aliases present in the devices variable for any host in the [osds] inventory group.

osd_preconfig.yml: This playbook applies some kernel level settings to the host to improve the performance of osds and the cluster.

osd_keyring.yml: This playbook creates and places the bootstrap osd keyring in the required folder if it does not exist there already.

generate_osd_vars.yml: This playbook executes a script designed to gather the drive aliases of drives on the host for future use in creatingthe osds. This playbook will create host_vars variable files for any osd nodes that don't have them already and populate them with the detected drive aliases that are to be used as osds. If the variables for dedicated devices have already been set in that host's host_vars file, this playbook will not add the dedicated devices to this list of devices to prevent issues.

alias_script.yml: This script retrieves the generate_osd_vars.sh script from the ansible host and copies it to other nodes in the cluster. This script is necessary to run the generate_osd_vars.yml playbook.

add_osds.yml: This playbook gathers the drive aliases listed in the host_vars files, resolves them back into their proper file paths, and uses these file paths to generate osds. If there are dedicated devices set for that host, then the playbook will use ceph-volume lvm batch to provision them.

add_mdss.yml: This playbook configures any hosts set in the inventory.ini file in the [mdss] group as a mds node.

create_filesystem.yml: This playbook creates a basic iteration of cephfs. It creates a data and metadata pool for cephfs and then creates the filesystem using the two pools. Note: This playbook will only run if there's at least two mdss' in the inventory file. 

add_additional_pools.yml This playbook will add additional pools to the filesystem. IT can add three at a time and requires the additional_pools variable in /group_vars/all.yml to be defined for it to execute.

ceph_aliases.yml: This is an alternate version of the generate_osd_vars.yml playbook meant for hosts that do not have 45drives tools installed. This playbook will copy the generate_osd_vars.sh script to every host's /etc directory and then execute a modified generate_osd_vars.yml playbook that executes the bash script on each host from the /etc directory.

add_rgws.yml: This playbook will configure every node in the rgws inventory group as a RADOS gateway node.

HOW TO USE

In this revision of the playbook, do the following:

Install the preconfiguration tools. Run these commands on each host that is to be part of the cluster. 

curl -sSL https://repo.45drives.com/setup | sudo bash 

Rocky Linux:
- curl -LO https://scripts.45drives.com/rocky-preconfig.sh
- sudo bash rocky-preconfig.sh

Ubuntu:
- curl -LO https://scripts.45drives.com/ubuntu-preconfig.sh
- sudo bash ubuntu-preconfig.sh

Ensure that the host running Ansible can passwordless SSH into every other node.

Add the nodes that will be used in the cluster to the inventory.ini file following the required format "HOSTNAME ansible_host=IP_ADDRESS"

Edit the group_vars/all.yml file to reflect details including public_network, cluster_network, and mon_initial_hosts.

If dedicated devices are to be used, add/edit the host_vars/HOSTNAME.yml file to list the aliases of the dedicated devices to be used on that host.

Ensure that only new OSDs are being added when running ceph_core.yml. If no new OSDs are being added on execution, remove or comment out all entries in the [osds] group in the inventory file.

Run the playbooks in the following order:
If 45Drives Tools was installed:
- generate_osd_vars.yml
- ceph_initial_setup.yml
- ceph_core.yml
- cephfs_setup.yml
- ceph_rgw.yml (if applicable)

If 45Drives Tools was NOT installed:
- ceph_drive_aliases.yml
- ceph_initial_setup.yml
- ceph_core.yml
- cephfs_setup.yml
- ceph_rgw.yml (if applicable)


TO-DO LIST

Here's the current list of tasks to be completed for future iterations of the playbook:
- Further testing of RGW tasks.
- Bug Testing and additional safety measures
- Playbook structure overhauls based on user feedback.
